/* eslint-disable react/jsx-handler-names */
import React, { Component } from 'react';
import PropTypes from 'prop-types';
import { InputText } from 'primereact/inputtext';
import Checkbox from './Checkbox';

export class MultiSelectHeader extends Component {
	constructor() {
		super();
		this.onFilter = this.onFilter.bind(this);
		this.onToggleAll = this.onToggleAll.bind(this);
	}

	onFilter(event) {
		const { onFilter } = this.props;
		if (onFilter) {
			onFilter({
				originalEvent: event,
				query: event.target.value,
			});
		}
	}

	onToggleAll(event) {
		const { allChecked, onToggleAll } = this.props;
		if (onToggleAll) {
			onToggleAll({
				originalEvent: event,
				checked: allChecked,
			});
		}
	}

	renderFilterElement() {
		const { filter, filterValue, label, parentId } = this.props;
		if (filter) {
			return (
				<div className="p-multiselect-filter-container">
					<InputText
						type="text"
						role="textbox"
						value={filterValue}
						onChange={this.onFilter}
						className="p-inputtext p-component"
					/>
					<span className="p-multiselect-filter-icon pi pi-search" />
				</div>
			);
		} else {
			return (
				<label id={`${parentId}-header-label`} htmlFor={`${parentId}-header`} className="p-label">
					{label}
				</label>
			);
		}
	}

	render() {
		const { allChecked, label, onClose, parentId } = this.props;
		const filterElement = this.renderFilterElement();
		return (
			<div className="p-multiselect-header">
				{filterElement}
				<Checkbox
					className="float-left"
					inputId={`${parentId}-header`}
					checked={allChecked}
					onChange={this.onToggleAll}
					role="checkbox"
					aria-checked={allChecked}
					ariaLabel={label}
					ariaLabelledBy={`${parentId}-header-label`}
				/>
				<button
					type="button"
					className="p-multiselect-close p-link"
					onClick={onClose}
					aria-label="Zakończ zaznaczanie"
				>
					<span className="p-multiselect-close-icon pi pi-times" />
				</button>
			</div>
		);
	}
}

MultiSelectHeader.defaultProps = {
	filter: false,
	filterValue: null,
	onFilter: null,
	onClose: null,
	onToggleAll: null,
	allChecked: false,
};

MultiSelectHeader.propTypes = {
	allChecked: PropTypes.bool,
	filter: PropTypes.bool,
	filterValue: PropTypes.string,
	label: PropTypes.string,
	onClose: PropTypes.func,
	onFilter: PropTypes.func,
	onToggleAll: PropTypes.func,
	parentId: PropTypes.string.isRequired,
};

export default MultiSelectHeader;
