/* eslint-disable react/jsx-handler-names */
import React, { Component } from 'react';
import PropTypes from 'prop-types';
import classNames from 'classnames';

export class MultiSelectItem extends Component {
	constructor() {
		super();
		this.onClick = this.onClick.bind(this);
		this.onKeyDown = this.onKeyDown.bind(this);
	}

	onClick(event) {
		const { onClick, option } = this.props;
		if (onClick) {
			onClick({
				originalEvent: event,
				option,
			});
		}

		event.preventDefault();
	}

	onKeyDown(event) {
		const { onKeyDown, option } = this.props;
		if (onKeyDown) {
			onKeyDown({
				originalEvent: event,
				option,
			});
		}
	}

	render() {
		const {
			label,
			option,
			optionDisabledField,
			optionDisabledReversed,
			optionsDisabled,
			selected,
			tabIndex,
			template,
		} = this.props;
		const className = classNames('p-multiselect-item', { 'p-highlight': selected });
		const checkboxClassName = classNames('p-checkbox-box p-component', {
			'p-highlight': selected,
		});
		const checkboxIcon = classNames('p-checkbox-icon p-c', { 'pi pi-check': selected });
		const content = template ? template(option) : label;
		const disabled =
			optionsDisabled &&
			(optionDisabledReversed ? !option[optionDisabledField] : option[optionDisabledField]);
		console.log('item', disabled, optionsDisabled, option[optionDisabledField]);
		return (
			<li
				className={className}
				onClick={disabled ? null : this.onClick}
				tabIndex={tabIndex}
				onKeyDown={this.onKeyDown}
				role="option"
				aria-selected={selected}
				aria-label={content}
			>
				<div className={'p-checkbox p-component'}>
					<div className={`${checkboxClassName} ${disabled ? 'p-disabled' : ''}`}>
						<span className={checkboxIcon} />
					</div>
				</div>
				<span>{content}</span>
			</li>
		);
	}
}

MultiSelectItem.defaultProps = {
	option: null,
	label: null,
	selected: false,
	tabIndex: null,
	template: null,
	onClick: null,
	onKeyDown: null,
	optionDisabledField: 'disabled',
	optionDisabledReversed: false,
	optionsDisabled: false,
};

MultiSelectItem.propTypes = {
	label: PropTypes.string,
	onClick: PropTypes.func,
	onKeyDown: PropTypes.func,
	option: PropTypes.object,
	optionDisabledField: PropTypes.string,
	optionDisabledReversed: PropTypes.bool,
	optionsDisabled: PropTypes.bool,
	selected: PropTypes.bool,
	tabIndex: PropTypes.string,
	template: PropTypes.func,
};

export default MultiSelectItem;
